@@ -27,7 +27,8 @@
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
-
+#include <sys/time.h>
+#include <sys/resource.h>
 
 #ifdef __FreeBSD__
 // Grmbl  Why is this needed?  We don't use readv/writev
@@ -53,6 +54,7 @@
 #include "md5.h"
 #include "util.h"
 #include "services/util.h"
+#include "pipes.h"
 
 #ifndef O_LARGEFILE
 #define O_LARGEFILE 0
@@ -85,7 +87,7 @@ parse_icecc_version(const string &target_platform, const string &prefix)
 
     // free after the C++-Programming-HOWTO
     string::size_type lastPos = icecc_version.find_first_not_of(',', 0);
-    string::size_type pos     = icecc_version.find_first_of(',', lastPos);
+    string::size_type pos     = icecc_version.find(',', lastPos);
     bool def_targets = icecc_version.find('=') != string::npos;
 
     list<string> platforms;
@@ -104,7 +106,7 @@ parse_icecc_version(const string &target_platform, const string &prefix)
         // Skip delimiters.  Note the "not_of"
         lastPos = icecc_version.find_first_not_of(',', pos);
         // Find next "non-delimiter"
-        pos = icecc_version.find_first_of(',', lastPos);
+        pos = icecc_version.find(',', lastPos);
 
         if (def_targets) {
             colon = version.find('=');
@@ -164,17 +166,18 @@ rip_out_paths(const Environments &envs, map<string, string> &version_map, map<st
 
     Environments env2;
 
-    static const char *suffs[] = { ".tar.bz2", ".tar.gz", ".tar", ".tgz", NULL };
+    static const char *suffs[] = { ".tar.xz", ".tar.zst", ".tar.bz2", ".tar.gz", ".tar", ".tgz", nullptr };
 
     string versfile;
 
-    for (Environments::const_iterator it = envs.begin(); it != envs.end(); ++it) {
-        for (int i = 0; suffs[i] != NULL; i++)
-            if (endswith(it->second, suffs[i], versfile)) {
-                versionfile_map[it->first] = it->second;
+    // host platform + filename
+    for (const std::pair<std::string, std::string> &env : envs) {
+        for (int i = 0; suffs[i] != nullptr; i++)
+            if (endswith(env.second, suffs[i], versfile)) {
+                versionfile_map[env.first] = env.second;
                 versfile = find_basename(versfile);
-                version_map[it->first] = versfile;
-                env2.push_back(make_pair(it->first, versfile));
+                version_map[env.first] = versfile;
+                env2.push_back(make_pair(env.first, versfile));
             }
     }
 
@@ -197,11 +200,17 @@ get_absfilename(const string &_file)
         file = _file;
     }
 
-    string::size_type idx = file.find("/..");
+    string dots = "/../";
+    string::size_type idx = file.find(dots);
 
     while (idx != string::npos) {
-        file.replace(idx, 3, "/");
-        idx = file.find("/..");
+        if (idx == 0) {
+            file.replace(0, dots.length(), "/");
+        } else {
+          string::size_type slash = file.rfind('/', idx - 1);
+          file.replace(slash, idx-slash+dots.length(), "/");
+        }
+        idx = file.find(dots);
     }
 
     idx = file.find("/./");
@@ -221,9 +230,21 @@ get_absfilename(const string &_file)
     return file;
 }
 
+static int get_niceness()
+{
+    errno = 0;
+    int niceness = getpriority( PRIO_PROCESS, getpid());
+    if( niceness == -1 && errno != 0 )
+        niceness = 0;
+    return niceness;
+}
+
 static UseCSMsg *get_server(MsgChannel *local_daemon)
 {
-    Msg *umsg = local_daemon->get_msg(4 * 60);
+    int timeout = 4 * 60;
+    if( get_niceness() > 0 ) // low priority jobs may take longer to get a slot assigned
+        timeout = 60 * 60;
+    Msg *umsg = local_daemon->get_msg( timeout );
 
     if (!umsg || umsg->type != M_USE_CS) {
         log_warning() << "reply was not expected use_cs " << (umsg ? (char)umsg->type : '0')  << endl;
@@ -247,7 +268,9 @@ static void check_for_failure(Msg *msg, MsgChannel *cserver)
     }
 }
 
-static void write_server_cpp(int cpp_fd, MsgChannel *cserver)
+// 'unlock_sending' = dcc_lock_host() is held when this is called, temporarily yield the lock
+// while doing network transfers
+static void write_fd_to_server(int fd, MsgChannel *cserver, bool unlock_sending = false)
 {
     unsigned char buffer[100000]; // some random but huge number
     off_t offset = 0;
@@ -258,16 +281,16 @@ static void write_server_cpp(int cpp_fd, MsgChannel *cserver)
         ssize_t bytes;
 
         do {
-            bytes = read(cpp_fd, buffer + offset, sizeof(buffer) - offset);
+            bytes = read(fd, buffer + offset, sizeof(buffer) - offset);
 
-            if (bytes < 0 && (errno == EINTR || errno == EAGAIN)) {
+            if (bytes < 0 && (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK)) {
                 continue;
             }
 
             if (bytes < 0) {
-                log_perror("reading from cpp_fd");
-                close(cpp_fd);
-                throw client_error(16, "Error 16 - error reading local cpp file");
+                log_perror("write_fd_to_server() reading from fd");
+                close(fd);
+                throw client_error(16, "Error 16 - error reading local file");
             }
 
             break;
@@ -277,6 +300,13 @@ static void write_server_cpp(int cpp_fd, MsgChannel *cserver)
 
         if (!bytes || offset == sizeof(buffer)) {
             if (offset) {
+                // If write_fd_to_server() is called for sending preprocessed data,
+                // the dcc_lock_host() lock is held to limit the number cpp invocations
+                // to the cores available to prevent overload. But that would
+                // essentially also limit network transfers, so temporarily yield and
+                // reaquire again.
+                if(unlock_sending)
+                    dcc_unlock();
                 FileChunkMsg fcmsg(buffer, offset);
 
                 if (!cserver->send_msg(fcmsg)) {
@@ -286,13 +316,22 @@ static void write_server_cpp(int cpp_fd, MsgChannel *cserver)
                     log_error() << "write of source chunk to host "
                                 << cserver->name.c_str() << endl;
                     log_perror("failed ");
-                    close(cpp_fd);
+                    close(fd);
                     throw client_error(15, "Error 15 - write to host failed");
                 }
 
                 uncompressed += fcmsg.len;
                 compressed += fcmsg.compressed;
                 offset = 0;
+                if(unlock_sending)
+                {
+                    if(!dcc_lock_host())
+                    {
+                        log_error() << "can't reaquire lock for local cpp" << endl;
+                        close(fd);
+                        throw client_error(32, "Error 32 - lock failed");
+                    }
+                }
             }
 
             if (!bytes) {
@@ -305,7 +344,7 @@ static void write_server_cpp(int cpp_fd, MsgChannel *cserver)
         trace() << "sent " << compressed << " bytes (" << (compressed * 100 / uncompressed) <<
                 "%)" << endl;
 
-    if ((-1 == close(cpp_fd)) && (errno != EBADF)){
+    if ((-1 == close(fd)) && (errno != EBADF)){
         log_perror("close failed");
     }
 }
@@ -322,7 +361,7 @@ static void receive_file(const string& output_file, MsgChannel* cserver)
         throw client_error(31, "Error 31 - " + errmsg);
     }
 
-    Msg* msg = 0;
+    Msg* msg = nullptr;
     size_t uncompressed = 0;
     size_t compressed = 0;
 
@@ -403,7 +442,7 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
 
     int status = 255;
 
-    MsgChannel *cserver = 0;
+    MsgChannel *cserver = nullptr;
 
     try {
         cserver = Service::createChannel(hostname, port, 10);
@@ -427,7 +466,7 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
             EnvTransferMsg msg(job.targetPlatform(), job.environmentVersion());
 
             if (!cserver->send_msg(msg)) {
-                throw client_error(6, "Error 6 - send environment to remove failed");
+                throw client_error(6, "Error 6 - send environment to remote failed");
             }
 
             int env_fd = open(version_file.c_str(), O_RDONLY);
@@ -436,7 +475,7 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
                 throw client_error(5, "Error 5 - unable to open version file:\n\t" + version_file);
             }
 
-            write_server_cpp(env_fd, cserver);
+            write_fd_to_server(env_fd, cserver);
 
             if (!cserver->send_msg(EndMsg())) {
                 log_error() << "write of environment failed" << endl;
@@ -456,9 +495,9 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
                     if (!static_cast<VerifyEnvResultMsg*>(verify_msg)->ok) {
                         // The remote can't handle the environment at all (e.g. kernel too old),
                         // mark it as never to be used again for this environment.
-                        log_info() << "Host " << hostname
-                                   << " did not successfully verify environment."
-                                   << endl;
+                        log_warning() << "Host " << hostname
+                                      << " did not successfully verify environment."
+                                      << endl;
                         BlacklistHostEnvMsg blacklist(job.targetPlatform(),
                                                       job.environmentVersion(), hostname);
                         local_daemon->send_msg(blacklist);
@@ -493,7 +532,7 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
             log_block b("send compile_file");
 
             if (!cserver->send_msg(compile_file)) {
-                log_info() << "write of job failed" << endl;
+                log_warning() << "write of job failed" << endl;
                 throw client_error(9, "Error 9 - error sending file to remote");
             }
         }
@@ -501,11 +540,18 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
         if (!preproc_file) {
             int sockets[2];
 
-            if (pipe(sockets)) {
+            if (create_large_pipe(sockets) != 0) {
+                log_perror("build_remote_in pipe");
                 /* for all possible cases, this is something severe */
-                exit(errno);
+                throw client_error(32, "Error 18 - (fork error?)");
             }
 
+            if (!dcc_lock_host()) {
+                log_error() << "can't lock for local cpp" << endl;
+                return EXIT_DISTCC_FAILED;
+            }
+            HostUnlock hostUnlock; // automatic dcc_unlock()
+
             /* This will fork, and return the pid of the child.  It will not
                return for the child itself.  If it returns normally it will have
                closed the write fd, i.e. sockets[1].  */
@@ -516,8 +562,8 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
             }
 
             try {
-                log_block bl2("write_server_cpp from cpp");
-                write_server_cpp(sockets[0], cserver);
+                log_block bl2("write_fd_to_server from cpp");
+                write_fd_to_server(sockets[0], cserver, true /*yield lock*/);
             } catch (...) {
                 kill(cpp_pid, SIGTERM);
                 throw;
@@ -529,9 +575,17 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
 
             if (shell_exit_status(status) != 0) {   // failure
                 delete cserver;
-                cserver = 0;
+                cserver = nullptr;
                 log_warning() << "call_cpp process failed with exit status " << shell_exit_status(status) << endl;
-                return shell_exit_status(status);
+                // GCC's -fdirectives-only has a number of cases that it doesn't handle properly,
+                // so if in such mode preparing the source fails, try again recompiling locally.
+                // This will cause double error in case it is a real error, but it'll build successfully if
+                // it was just -fdirectives-only being broken. In other cases fail directly, Clang's
+                // -frewrite-includes is much more reliable than -fdirectives-only, so is GCC's plain -E.
+                if( !compiler_is_clang(job) && compiler_only_rewrite_includes(job))
+                    throw remote_error(103, "Error 103 - local cpp invocation failed, trying to recompile locally");
+                else
+                    return shell_exit_status(status);
             }
         } else {
             int cpp_fd = open(preproc_file, O_RDONLY);
@@ -540,12 +594,12 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
                 throw client_error(11, "Error 11 - unable to open preprocessed file");
             }
 
-            log_block cpp_block("write_server_cpp");
-            write_server_cpp(cpp_fd, cserver);
+            log_block cpp_block("write_fd_to_server preprocessed");
+            write_fd_to_server(cpp_fd, cserver);
         }
 
         if (!cserver->send_msg(EndMsg())) {
-            log_info() << "write of end failed" << endl;
+            log_warning() << "write of end failed" << endl;
             throw client_error(12, "Error 12 - failed to send file to remote");
         }
 
@@ -574,18 +628,24 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
 
         if (status && crmsg->was_out_of_memory) {
             delete crmsg;
-            log_info() << "the server ran out of memory, recompiling locally" << endl;
+            log_warning() << "the server ran out of memory, recompiling locally" << endl;
             throw remote_error(101, "Error 101 - the server ran out of memory, recompiling locally");
         }
 
         if (output) {
             if ((!crmsg->out.empty() || !crmsg->err.empty()) && output_needs_workaround(job)) {
                 delete crmsg;
-                log_info() << "command needs stdout/stderr workaround, recompiling locally" << endl;
-                log_info() << "(set ICECC_CARET_WORKAROUND=0 to override)" << endl;
+                log_warning() << "command needs stdout/stderr workaround, recompiling locally" << endl;
+                log_warning() << "(set ICECC_CARET_WORKAROUND=0 to override)" << endl;
                 throw remote_error(102, "Error 102 - command needs stdout/stderr workaround, recompiling locally");
             }
 
+            if (crmsg->err.find("file not found") != string::npos) {
+                delete crmsg;
+                log_warning() << "remote is missing file, recompiling locally" << endl;
+                throw remote_error(104, "Error 104 - remote is missing file, recompiling locally");
+            }
+
             ignore_result(write(STDOUT_FILENO, crmsg->out.c_str(), crmsg->out.size()));
 
             if (colorify_wanted(job)) {
@@ -595,7 +655,7 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
             }
 
             if (status && (crmsg->err.length() || crmsg->out.length())) {
-                log_error() << "Compiled on " << hostname << endl;
+                log_info() << "Compiled on " << hostname << endl;
             }
         }
 
@@ -607,7 +667,7 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
         if (status == 0) {
             receive_file(job.outputFile(), cserver);
             if (have_dwo_file) {
-                string dwo_output = job.outputFile().substr(0, job.outputFile().find_last_of('.')) + ".dwo";
+                string dwo_output = job.outputFile().substr(0, job.outputFile().rfind('.')) + ".dwo";
                 receive_file(dwo_output, cserver);
             }
         }
@@ -622,7 +682,7 @@ static int build_remote_int(CompileJob &job, UseCSMsg *usecs, MsgChannel *local_
                 delete msg;
             }
             delete cserver;
-            cserver = 0;
+            cserver = nullptr;
         }
 
         throw;
@@ -691,7 +751,7 @@ maybe_build_local(MsgChannel *local_daemon, UseCSMsg *usecs, CompileJob &job,
         CompileFileMsg compile_file(&job);
 
         if (!local_daemon->send_msg(compile_file)) {
-            log_info() << "write of job failed" << endl;
+            log_warning() << "write of job failed" << endl;
             throw client_error(29, "Error 29 - write of job failed");
         }
 
@@ -699,11 +759,11 @@ maybe_build_local(MsgChannel *local_daemon, UseCSMsg *usecs, CompileJob &job,
 
         struct rusage ru;
 
-        gettimeofday(&begintv, 0);
+        gettimeofday(&begintv, nullptr);
 
         ret = build_local(job, local_daemon, &ru);
 
-        gettimeofday(&endtv, 0);
+        gettimeofday(&endtv, nullptr);
 
         // filling the stats, so the daemon can play proxy for us
         JobDoneMsg msg(job_id, ret, JobDoneMsg::FROM_SUBMITTER);
@@ -716,7 +776,7 @@ maybe_build_local(MsgChannel *local_daemon, UseCSMsg *usecs, CompileJob &job,
         if (!stat(job.outputFile().c_str(), &st)) {
             msg.out_uncompressed += st.st_size;
         }
-        if (!stat((job.outputFile().substr(0, job.outputFile().find_last_of('.')) + ".dwo").c_str(), &st)) {
+        if (!stat((job.outputFile().substr(0, job.outputFile().rfind('.')) + ".dwo").c_str(), &st)) {
             msg.out_uncompressed += st.st_size;
         }
 
@@ -750,9 +810,21 @@ static int minimalRemoteVersion( const CompileJob& job)
     return version;
 }
 
+static unsigned int requiredRemoteFeatures()
+{
+    unsigned int features = 0;
+    if (const char* icecc_env_compression = getenv( "ICECC_ENV_COMPRESSION" )) {
+        if( strcmp( icecc_env_compression, "xz" ) == 0 )
+            features = features | NODE_FEATURE_ENV_XZ;
+        if( strcmp( icecc_env_compression, "zstd" ) == 0 )
+            features = features | NODE_FEATURE_ENV_ZSTD;
+    }
+    return features;
+}
+
 int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &_envs, int permill)
 {
-    srand(time(0) + getpid());
+    srand(time(nullptr) + getpid());
 
     int torepeat = 1;
     bool has_split_dwarf = job.dwarfFissionEnabled();
@@ -800,8 +872,10 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
         GetCSMsg getcs(envs, fake_filename, job.language(), torepeat,
                        job.targetPlatform(), job.argumentFlags(),
                        preferred_host ? preferred_host : string(),
-                       minimalRemoteVersion(job));
+                       minimalRemoteVersion(job), requiredRemoteFeatures(),
+                       get_niceness());
 
+        trace() << "asking for host to use" << endl;
         if (!local_daemon->send_msg(getcs)) {
             log_warning() << "asked for CS" << endl;
             throw client_error(24, "Error 24 - asked for CS");
@@ -815,7 +889,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
                 ret = build_remote_int(job, usecs, local_daemon,
                                        version_map[usecs->host_platform],
                                        versionfile_map[usecs->host_platform],
-                                       0, true);
+                                       nullptr, true);
         } catch(...) {
             delete usecs;
             throw;
@@ -824,10 +898,17 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
         delete usecs;
         return ret;
     } else {
-        char *preproc = 0;
+        char *preproc = nullptr;
         dcc_make_tmpnam("icecc", ".ix", &preproc, 0);
         const CharBufferDeleter preproc_holder(preproc);
         int cpp_fd = open(preproc, O_WRONLY);
+
+        if (!dcc_lock_host()) {
+            log_error() << "can't lock for local cpp" << endl;
+            return EXIT_DISTCC_FAILED;
+        }
+        HostUnlock hostUnlock; // automatic dcc_unlock()
+
         /* When call_cpp returns normally (for the parent) it will have closed
            the write fd, i.e. cpp_fd.  */
         pid_t cpp_pid = call_cpp(job, cpp_fd);
@@ -845,6 +926,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
             ::unlink(preproc);
             return shell_exit_status(status);
         }
+        dcc_unlock();
 
         char rand_seed[400]; // "designed to be oversized" (Levi's)
         sprintf(rand_seed, "-frandom-seed=%d", rand());
@@ -853,7 +935,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
         GetCSMsg getcs(envs, get_absfilename(job.inputFile()), job.language(), torepeat,
                        job.targetPlatform(), job.argumentFlags(),
                        preferred_host ? preferred_host : string(),
-                       minimalRemoteVersion(job));
+                       minimalRemoteVersion(job), 0, get_niceness());
 
 
         if (!local_daemon->send_msg(getcs)) {
@@ -875,7 +957,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
 
         for (int i = 0; i < torepeat; i++) {
             jobs[i] = job;
-            char *buffer = 0;
+            char *buffer = nullptr;
 
             if (i) {
                 dcc_make_tmpnam("icecc", ".o", &buffer, 0);
@@ -958,7 +1040,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
                             log_perror("unlink outputFile failed") << "\t" << jobs[0].outputFile() << endl;
                         }
                         if (has_split_dwarf) {
-                            string dwo_file = jobs[0].outputFile().substr(0, jobs[0].outputFile().find_last_of('.')) + ".dwo";
+                            string dwo_file = jobs[0].outputFile().substr(0, jobs[0].outputFile().rfind('.')) + ".dwo";
                             if (-1 == ::unlink(dwo_file.c_str())){
                                 log_perror("unlink failed") << "\t" << dwo_file << endl;
                             }
@@ -979,7 +1061,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
                                (jobs[0].outputFile() + ".caught").c_str());
                         rename(preproc, (string(preproc) + ".caught").c_str());
                         if (has_split_dwarf) {
-                            string dwo_file = jobs[0].outputFile().substr(0, jobs[0].outputFile().find_last_of('.')) + ".dwo";
+                            string dwo_file = jobs[0].outputFile().substr(0, jobs[0].outputFile().rfind('.')) + ".dwo";
                             rename(dwo_file.c_str(), (dwo_file + ".caught").c_str());
                         }
                         exit_codes[0] = -1; // overwrite
@@ -991,7 +1073,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
                     log_perror("unlink failed") << "\t" << jobs[i].outputFile() << endl;
                 }
                 if (has_split_dwarf) {
-                    string dwo_file = jobs[i].outputFile().substr(0, jobs[i].outputFile().find_last_of('.')) + ".dwo";
+                    string dwo_file = jobs[i].outputFile().substr(0, jobs[i].outputFile().rfind('.')) + ".dwo";
                     if (-1 == ::unlink(dwo_file.c_str())){
                         log_perror("unlink failed") << "\t" << dwo_file << endl;
                     }
@@ -1003,7 +1085,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
                 log_perror("unlink failed") << "\t" << jobs[0].outputFile() << endl;
             }
             if (has_split_dwarf) {
-                string dwo_file = jobs[0].outputFile().substr(0, jobs[0].outputFile().find_last_of('.')) + ".dwo";
+                string dwo_file = jobs[0].outputFile().substr(0, jobs[0].outputFile().rfind('.')) + ".dwo";
                 if (-1 == ::unlink(dwo_file.c_str())){
                     log_perror("unlink failed") << "\t" << dwo_file << endl;
                 }
@@ -1014,7 +1096,7 @@ int build_remote(CompileJob &job, MsgChannel *local_daemon, const Environments &
                     log_perror("unlink failed") << "\t" << jobs[i].outputFile() << endl;
                 }
                 if (has_split_dwarf) {
-                    string dwo_file = jobs[i].outputFile().substr(0, jobs[i].outputFile().find_last_of('.')) + ".dwo";
+                    string dwo_file = jobs[i].outputFile().substr(0, jobs[i].outputFile().rfind('.')) + ".dwo";
                     if (-1 == ::unlink(dwo_file.c_str())){
                         log_perror("unlink failed") << "\t" << dwo_file << endl;
                     }
