@@ -93,6 +93,12 @@ public:
     int minimalHostVersion() const;
     void setMinimalHostVersion( int version );
 
+    unsigned int requiredFeatures() const;
+    void setRequiredFeatures(unsigned int features);
+
+    int niceness() const;
+    void setNiceness( int niceness );
+
 private:
     const unsigned int m_id;
     unsigned int m_localClientId;
@@ -117,6 +123,8 @@ private:
     std::string m_language; // for debugging
     std::string m_preferredHost; // for debugging daemons
     int m_minimalHostVersion; // minimal version required for the the remote server
+    unsigned int m_requiredFeatures; // flags the job requires on the remote server
+    int m_niceness; // nice priority (0-20)
 };
 
 #endif
