@@ -1,7 +1,7 @@
 #ifndef INCLUDES_H
 #define INCLUDES_H
 
-#include <stdlib.h>
+#include <string.h>
 #include <iostream>
 
 #endif
